# tinyrox says don't edit this manually, but it can't stop you!

export(audio_to_mel)
export(detect_language)
export(download_whisper_model)
export(list_downloaded_models)
export(list_whisper_models)
export(load_audio)
export(load_whisper_model)
export(model_exists)
export(serve)
export(transcribe)
export(whisper_config)
export(whisper_device)
export(whisper_dtype)
export(whisper_pipeline)
export(whisper_tokenizer)
export(whisper_tune_gc)

S3method(print,whisper_pipeline)

importFrom(stats,median)
importFrom(stats,setNames)
importFrom(utils,capture.output)
importFrom(utils,download.file)
importFrom(utils,read.csv)
