

This archive contains the distribution Whisper,
version 1.031:

  Handle Whisper fixed-size database files

This software is Copyright (c) 2013 by Original Copyright 2008 Orbitz WorldWide (python), Perl port 2013 Jean Stebens.

This is free software, licensed under:

  The Apache License, Version 2.0, January 2004


